/*
    Copyright (C) 2002 Kjetil S. Matheussen / Notam.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdlib.h>
#include <stdio.h>
#include <sched.h>
#include <sys/mman.h>

#include <vstserver.h>
#include "s_.h"
#include "processthread.h"

#include <errno.h>

/* Should have had a watchdog here... */

static void set_realtime(struct VSTClient *client){
    struct sched_param par;

    char *app = getenv("GIVERTCAP");
    if(app==NULL) app = "/usr/bin/givertcap";  
    system(app);

    par.sched_priority = (sched_get_priority_max(SCHED_FIFO) + sched_get_priority_min(SCHED_FIFO) ) / 2;

    if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
      fprintf(stderr,"VSTSERVER/PROCESS_thread: Unable to set SCHED_FIFO realtime priority.\n");
    }else{
      printf("VSTSERVER/PROCESS_thread: Realtime priority set for \"%s\".\n",client->name);
    }

    if(mlockall(MCL_FUTURE)==-1){
    	fprintf(stderr,"VSTSERVER/PROCESS_thread: Unable to lock memory. (%s)\n",strerror(errno));
    }
}



void *PROCESS_thread(void *arg){
  struct VSTClient *client=(struct VSTClient *)arg;

  while(C_acceptIncomingConnection(client->process_input)==false);

  if(client->realtime==1){
    set_realtime(client);
  }

  client->process_thread_status=1;

  for(;;){
    int sampleframes;
    int num_read=C_receive(client->process_input,&sampleframes,sizeof(int));
    if(num_read!=sizeof(int)){
      if(num_read==0){
	fprintf(stderr,"VSTSERVER/PROCESS_thread: Seems like client has died. Exiting.\n");
	return NULL;
      }else{
	fprintf(stderr,"VSTSERVER/PROCESS_thread: num_read not %d but %d.\n",sizeof(int),num_read);
      }
      continue;
    }

    if(sampleframes==0){
      printf("VSTSERVER/PROCESS_thread: received delete.\n");
      return NULL;
    }

    if(sampleframes>0){
      VSTS_process(client,sampleframes);
    }else{
      VSTS_processReplacing(client,-sampleframes);
    }
  }

  return NULL;
}

